"""template-specific forms/views/actions/components"""
from logilab.common.decorators import monkeypatch

from cubicweb.web import uicfg, formwidgets as fw
from cubicweb.web.views import basecontrollers

_afs = uicfg.autoform_section
_affk = uicfg.autoform_field_kwargs
_afs.tag_subject_of(('Expense', 'spent_for', '*'), 'main', 'attributes')
_afs.tag_subject_of(('Expense', 'spent_for', '*'), 'muledit', 'attributes')
_affk.tag_subject_of(('Expense', 'spent_for', '*'),
                     {'widget': fw.RestrictedAutoCompletionWidget(autocomplete_initfunc='get_concerned_by')})


@monkeypatch(basecontrollers.JSonController)
@basecontrollers.jsonize
def js_get_concerned_by(self):
    return self._cw.execute('DISTINCT Any W,R ORDERBY R WHERE W ref R').rows



"""specific actions for fresh template

:organization: Logilab
:copyright: 2008-2010 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
:contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
"""
__docformat__ = "restructuredtext en"

from cubicweb.selectors import implements
from cubicweb.web.action import Action

class AccountingAction(Action):
    __regid__ = 'accaction'
    __select__ = implements('Expense')
    title = _('generate accounting entries')

    def url(self):
        entity = self.cw_rset.get_entity(self.cw_row or 0, self.cw_col or 0)
        return entity.absolute_url(vid='accexpense')




# template's specific schema
from yams.buildobjs import RelationDefinition
from cubicweb.schema import RRQLExpression

class spent_for(RelationDefinition):
    subject = 'Expense'
    object = 'Workcase'
    cardinality = '?*'
    __permissions__ = {
        'read' : ('managers', 'users'),
        'add': ('managers', RRQLExpression('NOT (S in_state ST, ST name "accepted")')),
        'delete': ('managers', RRQLExpression('NOT (S in_state ST, ST name "accepted")')),
        }



"""this contains the template-specific entities' classes"""

from cubes.expense.entities import ExpenseLine as BaseExpenseLine

class ExpenseLine(BaseExpenseLine):

    @property
    def workcase(self):
        rql = 'Any R WHERE E has_lines EL, EL eid %(el)s, E spent_for W, W ref R'
        rset = self._cw.execute(rql, {'el': self.eid}, 'el')
        if rset:
            return rset[0][0]
        return None



# -*- coding: utf-8 -*-
# postcreate script. You could setup a workflow here for example

for login in (u'alf', u'syt', u'nico', u'jphc', u'ocy', u'auc', u'katia',
              u'graz', u'dede', u'juj', u'ludal', u'steph', u'arthur',
              u'david', u'joel', u'gaston', u'adim'):
    rql('INSERT CWUser E: E login %(login)s, E upassword %(login)s, E in_group G '
        'WHERE G name "users"', {'login' : login})
    rql('INSERT PaidByAccount P: P label %(label)s, P associated_to U WHERE U login %(login)s',
        {'label' : u"refund account - %s" % login, 'login': login})
    rql('INSERT PaidForAccount P: P label %(label)s', {'label' : u"charge account - %s" % login})


for label in (u'Logilab - CB Nicolas', u'Logilab - CB Alexandre', u'Logilab - CB Olivier',
              u'Logilab - Espèces'):
    rql('INSERT PaidByAccount P: P label %(label)s', {'label' : label})



"""template automatic tests"""

from logilab.common.testlib import unittest_main

from cubicweb.devtools.htmlparser import SaxOnlyValidator
from cubicweb.devtools.testlib import AutomaticWebTest

AutomaticWebTest.vid_validators.update({
    'accexpense': SaxOnlyValidator,
    'accentry': SaxOnlyValidator,
    })

if __name__ == '__main__':
    unittest_main()



# pylint: disable-msg=W0622
"""cubicweb-fresh application packaging information"""

modname = 'fresh'
distname = 'cubicweb-fresh'

numversion = (0, 5, 1)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
copyright = '''Copyright (c) 2008-2010 LOGILAB S.A. (Paris, FRANCE).
http://www.logilab.fr/ -- mailto:contact@logilab.fr'''

author = 'Logilab'
author_email = 'contact@logilab.fr'

short_desc = 'expense tracking application built on the CubicWeb framework'
long_desc = '''\
This cube provides a full-featured expense tracking application.
'''

ftp = ''
web = 'http://www.cubicweb.org/project/%s' % distname

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
    ]

pyversions = ['2.4']

__depends_cubes__ = {'expense': '>= 0.4.4',
                     'workcase': None,
                     }
__depends__ = {'cubicweb': '>= 3.6.0'}
for key, value in __depends_cubes__.items():
    __depends__['cubicweb-'+key] = value
__use__ = tuple(__depends_cubes__)
__recommend__ = ()

# packaging ###

from glob import glob
from os import listdir as _listdir
from os.path import join, isdir

#from cubicweb.devtools.pkginfo import get_distutils_datafiles
CUBES_DIR = join('share', 'cubicweb', 'cubes')
THIS_CUBE_DIR = join(CUBES_DIR, modname)

def listdir(dirpath):
    return [join(dirpath, fname) for fname in _listdir(dirpath)
            if fname[0] != '.' and not fname.endswith('.pyc')
            and not fname.endswith('~')
            and not isdir(join(dirpath, fname))]

from glob import glob
try:
    data_files = [
        # common files
        [THIS_CUBE_DIR, [fname for fname in glob('*.py') if fname != 'setup.py']],
        ]
    # check for possible extended cube layout
    for dirname in ('entities', 'views', 'sobjects', 'hooks', 'schema', 'data', 'i18n', 'migration', 'wdoc'):
        if isdir(dirname):
            data_files.append([join(THIS_CUBE_DIR, dirname), listdir(dirname)])
    # Note: here, you'll need to add subdirectories if you want
    # them to be included in the debian package
except OSError:
    # we are in an installed directory
    pass
